@@ -22,6 +22,7 @@
 
 #include "applet-struct.h"
 #include "applet-menu.h"  // start/stop
+#include "applet-apps.h"
 #include "applet-tree.h"
 
 #define CD_FOLDER_DEFAULT_ICON "folder"
@@ -94,12 +95,11 @@ static void add_image_to_menu_item (GtkWidget *image_menu_item,
 		gtk_image_set_from_icon_name (GTK_IMAGE (image), fallback_image_filename, GTK_ICON_SIZE_LARGE_TOOLBAR);
 
 	// insert the image in the menu-item
-	_gtk_image_menu_item_set_image (
-		GTK_IMAGE_MENU_ITEM (image_menu_item), image);
+	gldi_menu_item_set_image (image_menu_item, image);
 	gtk_widget_show (image);
 }
 
-static gchar * menu_escape_underscores_and_prepend (const char *text)
+/**static gchar * menu_escape_underscores_and_prepend (const char *text)
 {
 	GString    *escaped_text;
 	const char *src;
@@ -132,31 +132,7 @@ static gchar * menu_escape_underscores_and_prepend (const char *text)
 	}
 
 	return g_string_free (escaped_text, FALSE);
-}
-
-static void setup_menuitem (GtkWidget *menuitem,
-	const char  *title)
-{
-	GtkWidget *label;
-	gchar      *_title;
-
-	/* this creates a label with an invisible mnemonic */
-	label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
-	_title = menu_escape_underscores_and_prepend (title);
-	gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _title);
-	g_free (_title);
-	
-	gtk_label_set_pattern (GTK_LABEL (label), "");  // hide the _
-	
-	gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label), menuitem);
-	
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_widget_show (label);
-	
-	gtk_container_add (GTK_CONTAINER (menuitem), label);
-	
-	gtk_widget_show (menuitem);
-}
+}*/
 
 static GtkWidget * add_menu_separator (GtkWidget *menu)
 {
@@ -168,12 +144,13 @@ static GtkWidget * add_menu_separator (GtkWidget *menu)
 	return menuitem;
 }
 
-static GtkWidget * create_submenu_entry (GtkWidget          *menu,
+static GtkWidget * create_submenu_entry (GtkWidget *menu,
 	GMenuTreeDirectory *directory)
 {
-	GtkWidget *menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-		gmenu_tree_directory_get_name (directory));
+	if (gmenu_tree_directory_get_is_nodisplay (directory))
+		return NULL;
+	
+	GtkWidget *menuitem = gldi_menu_item_new (gmenu_tree_directory_get_name (directory), "");
 	
 	GIcon *pIcon = gmenu_tree_directory_get_icon (directory);  // transfer: None
 	add_image_to_menu_item (menuitem,
@@ -184,7 +161,7 @@ static GtkWidget * create_submenu_entry (GtkWidget          *menu,
 	return menuitem;
 }
 
-static void create_submenu (GtkWidget          *menu,
+static void create_submenu (GtkWidget *menu,
 	GMenuTreeDirectory *directory,
 	GMenuTreeDirectory *alias_directory)
 {
@@ -194,9 +171,11 @@ static void create_submenu (GtkWidget          *menu,
 		menuitem = create_submenu_entry (menu, alias_directory);
 	else
 		menuitem = create_submenu_entry (menu, directory);
+	if (!menuitem)
+		return;
 	
 	// create a sub-menu for it
-	GtkWidget *submenu = gtk_menu_new ();
+	GtkWidget *submenu = gldi_submenu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
 	
 	// populate the sub-menu with the directory
@@ -216,16 +195,23 @@ static void create_menuitem (GtkWidget *menu,
 	GMenuTreeEntry *entry,
 	GMenuTreeDirectory *alias_directory)
 {
-	// create an entry
+	// register the application
 	GDesktopAppInfo *pAppInfo = gmenu_tree_entry_get_app_info (entry);
+	cd_menu_register_app (pAppInfo);
+	
+	// ignore entry that are not shown in the menu
+	if (gmenu_tree_entry_get_is_excluded (entry))
+		return;
+	if (! g_app_info_should_show (G_APP_INFO (pAppInfo)))  // should_show = NoDisplay + OnlyShowIn
+		return;
+	
+	// create an entry
 	const gchar *cName = NULL;
 	if (alias_directory)
 		cName = gmenu_tree_directory_get_name (alias_directory);
 	if (!cName)
 		cName = g_app_info_get_name (G_APP_INFO (pAppInfo));
-	GtkWidget *menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-		cName);
+	GtkWidget *menuitem = gldi_menu_item_new (cName, "");
 	
 	const gchar *cComment = NULL;
 	if (alias_directory)
@@ -315,7 +301,6 @@ static void cd_populate_menu_from_directory (GtkWidget *menu, GMenuTreeDirectory
 	while ((next_type = gmenu_tree_iter_next (iter)) != GMENU_TREE_ITEM_INVALID)
 	{
 		gpointer item = NULL;
-		
 		switch (next_type)
 		{
 			case GMENU_TREE_ITEM_DIRECTORY:  // we suppose that unicity is assured.
@@ -373,7 +358,7 @@ static gchar ** _get_xdg_menu_dirs (void)
 {
 	const gchar *dirs = g_getenv ("XDG_CONFIG_DIRS");
 	if (! dirs || *dirs == '\0')
-		dirs = "/etc/xdg/menus";
+		dirs = "/etc/xdg";
 
 	return g_strsplit (dirs, ":", 0);
 }
@@ -452,7 +437,7 @@ static gchar * cd_find_menu_file (const gchar *cMenuFile)
 GMenuTree *cd_load_tree_from_file (const gchar *cMenuFile)
 {
 	gchar *cMenuFileName = cd_find_menu_file (cMenuFile);
-	GMenuTree *tree = gmenu_tree_new (cMenuFileName, GMENU_TREE_FLAGS_NONE);
+	GMenuTree *tree = gmenu_tree_new (cMenuFileName, GMENU_TREE_FLAGS_INCLUDE_NODISPLAY | GMENU_TREE_FLAGS_INCLUDE_EXCLUDED);  /// GMENU_TREE_FLAGS_INCLUDE_NODISPLAY
 	if (! gmenu_tree_load_sync (tree, NULL))  // this does all the heavy work of parsing the .menu and each desktop files.
 	{
 		g_object_unref (tree);
