@@ -22,6 +22,10 @@
 #include <math.h>
 
 #ifdef CD_UPOWER_AVAILABLE
+/* to access suspend/resume functionality on Upower 0.9
+ * even if we use first logind, we only use logind via DBus
+ */
+#define UPOWER_ENABLE_DEPRECATED
 #include <upower.h>
 #endif
 
@@ -39,6 +43,7 @@ static void cd_logout_shut_down (void);
 static void cd_logout_restart (void);
 static void cd_logout_suspend (void);
 static void cd_logout_hibernate (void);
+static void cd_logout_hybridSleep (void);
 static void cd_logout_close_session (void);
 static void cd_logout_switch_to_user (const gchar *cUser);
 static void cd_logout_switch_to_guest (void);
@@ -49,107 +54,122 @@ static GList *cd_logout_get_users_list (void);
 
 static void _display_menu (void);
 
-#define CD_LOGOUT_MESSAGE_SEPARATOR " | "
-
   ////////////////////
  /// CAPABILITIES ///
 ////////////////////
 
-static void _cd_logout_check_capabilities_async (CDSharedMemory *pSharedMemory)
+/*
+ * Check if the method (cMethod) exists and returns 'yes' or 'challenge':
+ * http://www.freedesktop.org/wiki/Software/systemd/logind/
+ *  If "yes" is returned the operation is supported and the user may execute the
+ *   operation without further authentication.
+ *  If "challenge" is returned the operation is available, but only after
+ *   authorization.
+ * If it exists, *bIsAble is modified and TRUE is returned.
+ */
+static gboolean _cd_logout_check_capabilities_logind (DBusGProxy *pProxy, const gchar *cMethod, gboolean *bIsAble)
 {
 	GError *error = NULL;
-
-	// get capabilities from UPower.
-	#ifdef CD_UPOWER_AVAILABLE
-	UpClient *pUPowerClient = up_client_new ();
-	up_client_get_properties_sync (pUPowerClient, NULL, &error);  // this function always returns false ... and it crashes the dock (Debian 6) ! :-O
-	if (error)
-	{
-		cd_warning ("UPower error: %s", error->message);
-		g_error_free (error);
-		error = NULL;
-	}
-	else
-	{
-		pSharedMemory->bCanHibernate = up_client_get_can_hibernate (pUPowerClient);
-		pSharedMemory->bCanSuspend = up_client_get_can_suspend (pUPowerClient);
-	}
-	g_object_unref (pUPowerClient);
-	#endif
-	
-	// get capabilities from ConsoleKit.
-	DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
-		"org.freedesktop.ConsoleKit",
-		"/org/freedesktop/ConsoleKit/Manager",
-		"org.freedesktop.ConsoleKit.Manager");
-	
-	dbus_g_proxy_call (pProxy, "CanRestart", &error,
+	gchar *cResult = NULL;
+	dbus_g_proxy_call (pProxy, cMethod, &error,
 		G_TYPE_INVALID,
-		G_TYPE_BOOLEAN, &pSharedMemory->bCanRestart,
+		G_TYPE_STRING, &cResult,
 		G_TYPE_INVALID);
 	if (!error)
 	{
-		pSharedMemory->iLoginManager = CD_CONSOLE_KIT;
-		
-		dbus_g_proxy_call (pProxy, "CanStop", &error,
-			G_TYPE_INVALID,
-			G_TYPE_BOOLEAN, &pSharedMemory->bCanStop,
-			G_TYPE_INVALID);
-		if (error)
-		{
-			cd_warning ("ConsoleKit error: %s", error->message);
-			g_error_free (error);
-			g_object_unref (pProxy);
-			return;
-		}
+		*bIsAble = (cResult && (strcmp (cResult, "yes") == 0
+		            || strcmp (cResult, "challenge") == 0));
+		g_free (cResult);
 	}
-	else  // console kit is probably not used on this system, try with Logind from Systemd
+	else
 	{
-		cd_debug ("ConsoleKit error: %s", error->message);
+		cd_debug ("Logind error: %s", error->message);
 		g_error_free (error);
-		error = NULL;
+		return FALSE;
+	}
+	return TRUE;
+}
+
+static void _cd_logout_check_capabilities_async (CDSharedMemory *pSharedMemory)
+{
+	// test first with LoginD
+	DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
+		"org.freedesktop.login1",
+		"/org/freedesktop/login1",
+		"org.freedesktop.login1.Manager");
+
+	const gchar *cLogindMethods[] = {"CanPowerOff", "CanReboot", "CanSuspend", "CanHibernate", "CanHybridSleep", NULL};
+	gboolean *bCapabilities[] = {&pSharedMemory->bCanStop,
+		&pSharedMemory->bCanRestart, &pSharedMemory->bCanSuspend,
+		&pSharedMemory->bCanHibernate, &pSharedMemory->bCanHybridSleep};
+
+	if (pProxy && _cd_logout_check_capabilities_logind (pProxy, cLogindMethods[0], bCapabilities[0]))
+	{
+		pSharedMemory->iLoginManager = CD_LOGIND;
+		for (int i = 1; cLogindMethods[i] != NULL; i++)
+			_cd_logout_check_capabilities_logind (pProxy, cLogindMethods[i], bCapabilities[i]);
+
+		g_object_unref (pProxy);
+	}
+	else // then check with ConsoleKit and UPower
+	{
+		GError *error = NULL;
+
+		// get capabilities from UPower: hibernate and suspend
+		#ifdef CD_UPOWER_AVAILABLE
+		UpClient *pUPowerClient = up_client_new ();
+		if (pUPowerClient)
+		{
+			up_client_get_properties_sync (pUPowerClient, NULL, &error);  // this function always returns false ... and it crashes the dock (Debian 6) ! :-O
+			if (error)
+			{
+				cd_warning ("UPower error: %s", error->message);
+				g_error_free (error);
+				error = NULL;
+			}
+			else
+			{
+				pSharedMemory->bCanHibernate = up_client_get_can_hibernate (pUPowerClient);
+				pSharedMemory->bCanSuspend = up_client_get_can_suspend (pUPowerClient);
+				g_object_unref (pUPowerClient); // not do that if there is an error to avoid: double free or corruption
+			}
+		}
+		#endif
 		
+		// get capabilities from ConsoleKit.: reboot and poweroff
 		pProxy = cairo_dock_create_new_system_proxy (
-			"org.freedesktop.login1",
-			"/org/freedesktop/login1",
-			"org.freedesktop.login1.Manager");
+			"org.freedesktop.ConsoleKit",
+			"/org/freedesktop/ConsoleKit/Manager",
+			"org.freedesktop.ConsoleKit.Manager");
 		
-		gchar *res;
-		dbus_g_proxy_call (pProxy, "CanReboot", &error,
+		dbus_g_proxy_call (pProxy, "CanRestart", &error,
 			G_TYPE_INVALID,
-			G_TYPE_STRING, &res,
+			G_TYPE_BOOLEAN, &pSharedMemory->bCanRestart,
 			G_TYPE_INVALID);
 		if (!error)
 		{
-			pSharedMemory->iLoginManager = CD_SYSTEMD;
+			pSharedMemory->iLoginManager = CD_CONSOLE_KIT;
 			
-			pSharedMemory->bCanRestart = (res && strcmp (res, "yes") == 0);
-			g_free (res);
-			
-			dbus_g_proxy_call (pProxy, "CanPowerOff", &error,
+			dbus_g_proxy_call (pProxy, "CanStop", &error,
 				G_TYPE_INVALID,
-				G_TYPE_STRING, &res,
+				G_TYPE_BOOLEAN, &pSharedMemory->bCanStop,
 				G_TYPE_INVALID);
 			if (error)
 			{
-				cd_warning ("Logind error: %s", error->message);
+				cd_warning ("ConsoleKit error: %s", error->message);
 				g_error_free (error);
-				g_object_unref (pProxy);
-				return;
+				/*g_object_unref (pProxy); // should not happen but lets check guest account
+				return;*/
 			}
-			pSharedMemory->bCanStop = (res && strcmp (res, "yes") == 0);
-			g_free (res);
 		}
 		else
 		{
-			cd_debug ("Logind error: %s", error->message);
+			cd_debug ("ConsoleKit error: %s", error->message);
 			g_error_free (error);
-			error = NULL;
 		}
-		
+		g_object_unref (pProxy);
 	}
-	g_object_unref (pProxy);
-	
+
 	// get capabilities from DisplayManager
 	const gchar *seat = g_getenv ("XDG_SEAT_PATH");
 	if (seat)  // else, we could possibly get it by: ck -> GetCurrentSession -> session -> GetSeatId
@@ -174,12 +194,15 @@ static gboolean _cd_logout_got_capabilities (CDSharedMemory *pSharedMemory)
 	// fetch the capabilities.
 	myData.bCapabilitiesChecked = TRUE;
 	myData.bCanHibernate = pSharedMemory->bCanHibernate;
+	myData.bCanHybridSleep = pSharedMemory->bCanHybridSleep;
 	myData.bCanSuspend = pSharedMemory->bCanSuspend;
 	myData.bCanRestart = pSharedMemory->bCanRestart;
 	myData.bCanStop = pSharedMemory->bCanStop;
 	myData.bHasGuestAccount = pSharedMemory->bHasGuestAccount;
 	myData.iLoginManager = pSharedMemory->iLoginManager;
-	cd_debug ("capabilities: %d; %d; %d; %d; %d", myData.bCanHibernate, myData.bCanSuspend, myData.bCanRestart, myData.bCanStop, myData.bHasGuestAccount);
+	cd_debug ("capabilities: %d; %d; %d; %d; %d; %d", myData.bCanHibernate,
+		myData.bCanHybridSleep, myData.bCanSuspend, myData.bCanRestart,
+		myData.bCanStop, myData.bHasGuestAccount);
 	
 	// display the menu that has been asked beforehand.
 	_display_menu ();
@@ -237,9 +260,10 @@ static void _switch_to_user (GtkMenuItem *menu_item, gchar *cUserName)
 		cd_logout_switch_to_guest ();
 	}
 }
+static GtkWidget *pShutdownMenuItem;
 static GtkWidget *_build_menu (void)
 {
-	GtkWidget *pMenu = gtk_menu_new ();
+	GtkWidget *pMenu = gldi_menu_new (myIcon);
 	
 	GtkWidget *pMenuItem;
 
@@ -247,8 +271,9 @@ static GtkWidget *_build_menu (void)
 	cImagePath = _check_icon ("system-shutdown", myData.iDesiredIconSize);
 	pMenuItem = CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Shut down"), cImagePath ? cImagePath : MY_APPLET_SHARE_DATA_DIR"/system-shutdown.svg", cd_logout_shut_down, pMenu);
 	g_free (cImagePath);
-	if (!myData.bCanStop && ! myConfig.cUserAction2)
+	if (!myData.bCanStop && ! myConfig.cUserActionShutdown)
 		gtk_widget_set_sensitive (pMenuItem, FALSE);
+	pShutdownMenuItem = pMenuItem;
 	
 	cImagePath = _check_icon (GTK_STOCK_REFRESH, myData.iDesiredIconSize);
 	pMenuItem = CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Restart"), cImagePath ? cImagePath : MY_APPLET_SHARE_DATA_DIR"/system-restart.svg", cd_logout_restart, pMenu);
@@ -258,11 +283,17 @@ static GtkWidget *_build_menu (void)
 	
 	cImagePath = _check_icon ("sleep", myData.iDesiredIconSize);
 	pMenuItem = CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Hibernate"), cImagePath ? cImagePath : MY_APPLET_SHARE_DATA_DIR"/system-hibernate.svg", cd_logout_hibernate, pMenu);
-	g_free (cImagePath);
 	gtk_widget_set_tooltip_text (pMenuItem, D_("Your computer will not consume any energy."));
 	if (!myData.bCanHibernate)
 		gtk_widget_set_sensitive (pMenuItem, FALSE);
 	
+	if (myData.bCanHybridSleep)
+	{
+		pMenuItem = CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Hybrid Sleep"), cImagePath ? cImagePath : MY_APPLET_SHARE_DATA_DIR"/system-hibernate.svg", cd_logout_hybridSleep, pMenu);
+		gtk_widget_set_tooltip_text (pMenuItem, D_("Your computer will still consume a small amount of energy but after some time, the computer will suspend to disk and not consume any energy."));
+	}
+	g_free (cImagePath);
+	
 	cImagePath = _check_icon ("clock", myData.iDesiredIconSize);
 	pMenuItem = CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Suspend"), cImagePath ? cImagePath : MY_APPLET_SHARE_DATA_DIR"/system-suspend.svg", cd_logout_suspend, pMenu);
 	g_free (cImagePath);
@@ -346,19 +377,7 @@ static void _display_menu (void)
 	CD_APPLET_POPUP_MENU_ON_MY_ICON (pMenu);
 	
 	// select the first (or last) item, which corresponds to the 'shutdown' action.
-	if ((myDock && myDock->container.bIsHorizontal && ! myDock->container.bDirectionUp)  // on the top, we inverse the menu
-		|| (myDesklet && myDesklet->container.iWindowPositionY < (g_desktopGeometry.Xscreen.height / 2)))
-	{
-		GList *children = gtk_container_get_children (GTK_CONTAINER (pMenu));
-		GList *last_child = g_list_last (children);
-		if (last_child)
-			gtk_menu_shell_select_item (GTK_MENU_SHELL (pMenu), last_child->data);
-		g_list_free (children);
-	}
-	else
-	{
-		gtk_menu_shell_select_first (GTK_MENU_SHELL (pMenu), FALSE);  // must be done here, after the menu has been realized.
-	}
+	gtk_menu_shell_select_item (GTK_MENU_SHELL (pMenu), pShutdownMenuItem);
 }
 
 
@@ -387,7 +406,7 @@ static gboolean _timer (gpointer data)
 		CD_APPLET_SET_QUICK_INFO_ON_MY_ICON_PRINTF ("%dmn", (int) ceil ((double)(myConfig.iShutdownTime - t_cur) / 60.));
 		CD_APPLET_REDRAW_MY_ICON;
 		if (t_cur >= myConfig.iShutdownTime - 60)
-			cairo_dock_show_temporary_dialog_with_icon (D_("Your computer will shut-down in 1 minute."), myIcon, myContainer, 8000, "same icon");
+			gldi_dialog_show_temporary_with_icon (D_("Your computer will shut-down in 1 minute."), myIcon, myContainer, 8000, "same icon");
 	}
 	CD_APPLET_LEAVE (TRUE);
 	
@@ -434,7 +453,7 @@ static void _on_program_shutdown (int iClickedButton, GtkWidget *pInteractiveWid
 }
 void cd_logout_program_shutdown (void)
 {
-	cairo_dock_show_dialog_with_value (D_("Choose in how many minutes your PC will stop:"),
+	gldi_dialog_show_with_value (D_("Choose in how many minutes your PC will stop:"),
 		myIcon, myContainer,
 		"same icon",
 		30, 150,
@@ -472,32 +491,16 @@ static gchar * _get_reboot_message (void)
 	return (cMessage);
 }
 
-// it seems the file only contains a timestamp
-/* static const gchar * _get_logout_message (void) {}*/
-#define _get_logout_message() _("Your session needs to be restarted in order to update new configuration files.")
-
-// yes, it's not a good idea to reboot the computer before the end of the update ;)
-#define CD_LOGOUT_UPDATE_MESSAGE _("Please do that at the end of the update.")
-
 static void _notify_action_required (void)
 {
 	CD_APPLET_DEMANDS_ATTENTION ("pulse", 20);
-	cairo_dock_remove_dialog_if_any (myIcon);
+	gldi_dialogs_remove_on_icon (myIcon);
 
-	gchar *cName;
-	if (/*myData.bLogoutNeeded && */myData.bRebootNeeded)
-	{
-		gchar *cTmpName = g_strdup (myIcon->cName); // Icon's name contains the message for the reboot and then for the logout
-		gchar *cTmpPtr = g_strrstr (cTmpName, CD_LOGOUT_MESSAGE_SEPARATOR);
-		if (cTmpPtr)
-			*cTmpPtr = '\0';
-		cName = g_strdup_printf ("%s\n%s", cTmpName, CD_LOGOUT_UPDATE_MESSAGE);
-		g_free (cTmpName);
-	}
-	else
-		cName = g_strdup_printf ("%s\n%s", myIcon->cName, CD_LOGOUT_UPDATE_MESSAGE);
+	// it's not a good idea to reboot the computer before the end of the update ;)
+	gchar *cName = g_strdup_printf ("%s\n%s", myIcon->cName,
+		D_("Please do that at the end of the update."));
 
-	cairo_dock_show_temporary_dialog_with_icon (cName, myIcon, myContainer, 15e3, "same icon");
+	gldi_dialog_show_temporary_with_icon (cName, myIcon, myContainer, 15e3, "same icon");
 
 	g_free (cName);
 
@@ -526,7 +529,7 @@ static void _notify_action_required (void)
 static void _stop_notify_action_required (void)
 {
 	 // should not happen... mainly for the tests.
-	cairo_dock_remove_dialog_if_any (myIcon);
+	gldi_dialogs_remove_on_icon (myIcon);
 	if (myConfig.iRebootNeededImage == CD_DISPLAY_EMBLEM)
 		CD_APPLET_PRINT_OVERLAY_ON_MY_ICON (NULL, CAIRO_OVERLAY_UPPER_RIGHT);
 	else
@@ -534,93 +537,42 @@ static void _stop_notify_action_required (void)
 	CD_APPLET_STOP_DEMANDING_ATTENTION;
 }
 
-// complete the string with the right message
-static GString * _get_message (CDActionsNeededEnum iAction)
+void cd_logout_check_reboot_required (CairoDockFMEventType iEventType, const gchar *cURI)
 {
-	GString *sMessage = g_string_new ("");
-	if (iAction == CD_REBOOT_NEEDED || myData.bRebootNeeded) // reboot is now needed or reboot is still needed
-	{
-		myData.bRebootNeeded = TRUE;
-		gchar *cRebootMessage = _get_reboot_message ();
-		if (cRebootMessage && *cRebootMessage != '\0')
-			g_string_printf (sMessage, "%s", cRebootMessage);
-		g_free (cRebootMessage);
-	}
-/*
-	if (iAction == CD_LOGOUT_NEEDED || myData.bLogoutNeeded)
-	{
-		myData.bLogoutNeeded = TRUE;
-		g_string_append_printf (sMessage, "%s%s",
-			sMessage->len == 0 ? "" : CD_LOGOUT_MESSAGE_SEPARATOR,
-			_get_logout_message ());
-	}*/
-
-	return sMessage;
-}
-
-void cd_logout_check_reboot_logout_required (CairoDockFMEventType iEventType, const gchar *cURI, CDActionsNeededEnum iAction)
-{
-	// maybe check if logout or reboot message is already available  => merge both messages
-	GString *sMessage = NULL;
+	// maybe check if reboot message is already available
+	gchar *cMessage;
 
 	switch (iEventType)
 	{
 		case CAIRO_DOCK_FILE_MODIFIED: // new message
-		case CAIRO_DOCK_FILE_CREATED:  // reboot/logout required
-			sMessage = _get_message (iAction);
+		case CAIRO_DOCK_FILE_CREATED:  // reboot required
+			cMessage = _get_reboot_message ();
 		break;
 		
 		case CAIRO_DOCK_FILE_DELETED:  // reboot/logout no more required (shouldn't happen)
-			if (iAction == CD_REBOOT_NEEDED)
-				myData.bRebootNeeded = FALSE;
-			/*else if (iAction == CD_LOGOUT_NEEDED)
-				myData.bLogoutNeeded = FALSE;*/
-			sMessage = _get_message (CD_REMOVE_MESSAGE);
-			if (! myData.bRebootNeeded/* && ! myData.bLogoutNeeded*/)
-				_stop_notify_action_required (); // default icon
+			cMessage = NULL;
+			_stop_notify_action_required (); // default icon
 		break;
 		default:
 		break;
 	}
-	if (sMessage && sMessage->len > 0)
-		CD_APPLET_SET_NAME_FOR_MY_ICON (sMessage->str);
+	if (cMessage && *cMessage != '\0')
+		CD_APPLET_SET_NAME_FOR_MY_ICON (cMessage);
 	else
 		CD_APPLET_SET_NAME_FOR_MY_ICON (_get_default_message ());
 	if (iEventType == CAIRO_DOCK_FILE_CREATED)
 		_notify_action_required ();
 
-	g_string_free (sMessage, TRUE);
+	g_free (cMessage);
 }
 
 void cd_logout_check_reboot_required_init (void)
 {
 	if (g_file_test (CD_REBOOT_NEEDED_FILE, G_FILE_TEST_EXISTS))
 	{
-		cd_logout_check_reboot_logout_required (CAIRO_DOCK_FILE_CREATED, CD_REBOOT_NEEDED_FILE, CD_REBOOT_NEEDED);
+		cd_logout_check_reboot_required (CAIRO_DOCK_FILE_CREATED, CD_REBOOT_NEEDED_FILE);
 	}
 }
-/*
-const gchar *cd_logout_get_session_migration_filename (void)
-{
-	if (! myData.cSessionMigrationFileName)
-	{
-		// according to this bzr branch lp:session-migration
-		gchar *cSessionBaseName = g_strdup_printf ("session_migration-%s", g_getenv("DESKTOP_SESSION"));
-		myData.cSessionMigrationFileName = g_build_filename (g_get_user_data_dir (), cSessionBaseName, NULL); // ~/.local/share/session_migration-cairo-dock
-		g_free (cSessionBaseName);
-	}
-
-	return myData.cSessionMigrationFileName;
-}
-
-void cd_logout_check_logout_required_init (void)
-{
-	const gchar *cFileName = cd_logout_get_session_migration_filename ();
-	if (g_file_test (cFileName, G_FILE_TEST_EXISTS))
-	{
-		cd_logout_check_reboot_logout_required (CAIRO_DOCK_FILE_CREATED, cFileName, CD_LOGOUT_NEEDED);
-	}
-}*/
 
 
   ///////////////
@@ -646,7 +598,7 @@ static void _console_kit_action (const gchar *cAction)
 	g_object_unref (pProxy);
 }
 
-static void _systemd_action (const gchar *cAction)
+static void _logind_action (const gchar *cAction)
 {
 	GError *error = NULL;
 	DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
@@ -661,6 +613,14 @@ static void _systemd_action (const gchar *cAction)
 	if (error)
 	{
 		cd_warning ("Logind error: %s", error->message);
+		gchar *cMessage = g_strdup_printf ("%s %s\n%s",
+			D_("Logind has returned this error:"),
+			error->message,
+			D_("Please check that you can do this action\n"
+				"(e.g. you can't power the computer off if another session is active)"));
+		gldi_dialog_show_temporary_with_icon (cMessage, myIcon, myContainer,
+			15e3, "same icon");
+		g_free (cMessage);
 		g_error_free (error);
 	}
 	g_object_unref (pProxy);
@@ -693,12 +653,13 @@ static void _exec_action (int iClickedButton, GtkWidget *pInteractiveWidget, voi
 static void _demand_confirmation (const gchar *cMessage, const gchar *cIconStock, const gchar *cIconImage, void (*callback) (void))
 {
 	gchar *cImagePath = _check_icon (cIconStock, 32); // dialog
-	myData.pConfirmationDialog = cairo_dock_show_dialog_full (cMessage, myIcon, myContainer, 0, cImagePath ? cImagePath : cIconImage, NULL, (CairoDockActionOnAnswerFunc) _exec_action, callback, NULL);
+	myData.pConfirmationDialog = gldi_dialog_show (cMessage, myIcon, myContainer, 0, cImagePath ? cImagePath : cIconImage, NULL, (CairoDockActionOnAnswerFunc) _exec_action, callback, NULL);
 	g_free (cImagePath);
 }
 
 static void _shut_down (void)
 {
+	gldi_object_notify (&myModuleObjectMgr, NOTIFICATION_LOGOUT);
 	if (myData.bCanStop)
 	{
 		switch (myData.iLoginManager)
@@ -706,16 +667,16 @@ static void _shut_down (void)
 			case CD_CONSOLE_KIT:
 				_console_kit_action ("Stop");  // could use org.gnome.SessionManager.RequestShutdown, but it's not standard.
 			break;
-			case CD_SYSTEMD:
-				_systemd_action ("PowerOff");
+			case CD_LOGIND:
+				_logind_action ("PowerOff");
 			break;
 			default:
 			break;
 		}
 	}
-	else if (myConfig.cUserAction2)
+	else if (myConfig.cUserActionShutdown)
 	{
-		cairo_dock_launch_command (myConfig.cUserAction2);
+		cairo_dock_launch_command (myConfig.cUserActionShutdown);
 	}
 }
 static inline gchar *_info_msg (void)
@@ -731,7 +692,7 @@ static gboolean _auto_shot_down (gpointer data)
 	if (myData.iCountDown <= 0)
 	{
 		myData.iSidShutDown = 0;
-		cairo_dock_dialog_unreference (myData.pConfirmationDialog);
+		gldi_object_unref (GLDI_OBJECT(myData.pConfirmationDialog));
 		myData.pConfirmationDialog = NULL;
 		_shut_down ();
 		return FALSE;
@@ -741,7 +702,7 @@ static gboolean _auto_shot_down (gpointer data)
 		if (myData.pConfirmationDialog)  // paranoia
 		{
 			gchar *cMessage = _info_msg ();
-			cairo_dock_set_dialog_message (myData.pConfirmationDialog, cMessage);
+			gldi_dialog_set_message (myData.pConfirmationDialog, cMessage);
 			g_free (cMessage);
 		}
 		return TRUE;
@@ -768,6 +729,7 @@ void cd_logout_shut_down (void)
 
 static void _restart (void)
 {
+	gldi_object_notify (&myModuleObjectMgr, NOTIFICATION_LOGOUT);
 	if (myData.bCanRestart)
 	{
 		switch (myData.iLoginManager)
@@ -775,19 +737,20 @@ static void _restart (void)
 			case CD_CONSOLE_KIT:
 				_console_kit_action ("Restart");  // could use org.gnome.SessionManager.RequestShutdown, but it's not standard.
 			break;
-			case CD_SYSTEMD:
-				_systemd_action ("Reboot");
+			case CD_LOGIND:
+				_logind_action ("Reboot");
 				break;
 			default:
 				break;
 		}
 	}
-	else if (myConfig.cUserAction2)
+	else if (myConfig.cUserActionShutdown)
 	{
-		cairo_dock_launch_command (myConfig.cUserAction2);
+		cairo_dock_launch_command (myConfig.cUserActionShutdown);
 	}
 }
-void cd_logout_restart (void)
+
+static void cd_logout_restart (void)
 {
 	if (myConfig.bConfirmAction)
 	{
@@ -799,24 +762,40 @@ void cd_logout_restart (void)
 	}
 }
 
-void cd_logout_suspend (void)
+static void cd_logout_suspend (void)
 {
-	_upower_action (TRUE);
+	if (myData.iLoginManager == CD_LOGIND)
+		_logind_action ("Suspend");
+	else
+		_upower_action (TRUE);
 }
 
-void cd_logout_hibernate (void)
+static void cd_logout_hibernate (void)
 {
-	_upower_action (FALSE);
+	if (myData.iLoginManager == CD_LOGIND)
+		_logind_action ("Hibernate");
+	else
+		_upower_action (FALSE);
+}
+
+static void cd_logout_hybridSleep (void)
+{
+	if (myData.iLoginManager == CD_LOGIND)
+		_logind_action ("HybridSleep");
+	else
+		_upower_action (FALSE);
 }
 
 static void _logout (void)
 {
+	gldi_object_notify (&myModuleObjectMgr, NOTIFICATION_LOGOUT);
 	if (myConfig.cUserAction != NULL)
 		cairo_dock_launch_command (myConfig.cUserAction);
 	else  // SwitchToGreeter will only show the greeter, we want to close the session
 		cairo_dock_launch_command (MY_APPLET_SHARE_DATA_DIR"/logout.sh");
 }
-void cd_logout_close_session (void)  // could use org.gnome.SessionManager.Logout
+
+static void cd_logout_close_session (void)  // could use org.gnome.SessionManager.Logout
 {
 	/* Currently, cairo_dock_fm_logout displays to us a window from the DE
 	 * to confirm if we want to close the session or not. So there is a
@@ -835,14 +814,16 @@ void cd_logout_close_session (void)  // could use org.gnome.SessionManager.Logou
 			_logout ();
 		}
 	}
+	else
+		gldi_object_notify (&myModuleObjectMgr, NOTIFICATION_LOGOUT);
 }
 
 static void cd_logout_switch_to_user (const gchar *cUser)
 {
 	const gchar *seat = g_getenv ("XDG_SEAT_PATH");
+	GError *error = NULL;
 	if (seat)  // else, we could possibly get it by: ck -> GetCurrentSession -> session -> GetSeatId
 	{
-		GError *error = NULL;
 		DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
 			"org.freedesktop.DisplayManager",
 			seat,
@@ -854,8 +835,11 @@ static void cd_logout_switch_to_user (const gchar *cUser)
 			G_TYPE_INVALID);
 		if (error)
 		{
-			cd_warning ("DisplayManager error: %s", error->message);
+			cd_warning ("DisplayManager (LocalDisplay) error: %s", error->message);
 			g_error_free (error);
+			if (myConfig.cUserActionSwitchUser != NULL)
+				cairo_dock_launch_command_printf ("%s %s", NULL,
+					myConfig.cUserActionSwitchUser, cUser);
 		}
 		g_object_unref (pProxy);
 	}
@@ -865,9 +849,18 @@ static void cd_logout_switch_to_user (const gchar *cUser)
 			"org.gnome.DisplayManager",
 			"/org/gnome/DisplayManager/LocalDisplayFactory",
 			"org.gnome.DisplayManager.LocalDisplayFactory");
-		dbus_g_proxy_call_no_reply (pProxy, "SwitchToUser",
+		dbus_g_proxy_call (pProxy, "SwitchToUser", &error,
 			G_TYPE_STRING, cUser,
+			G_TYPE_INVALID,
 			G_TYPE_INVALID);  // we don't care the 'id' object path returned
+		if (error)
+		{
+			cd_warning ("DisplayManager error: %s", error->message);
+			g_error_free (error);
+			if (myConfig.cUserActionSwitchUser != NULL)
+				cairo_dock_launch_command_printf ("%s %s", NULL,
+					myConfig.cUserActionSwitchUser, cUser);
+		}
 		g_object_unref (pProxy);
 	}
 }
@@ -875,9 +868,9 @@ static void cd_logout_switch_to_user (const gchar *cUser)
 static void cd_logout_switch_to_guest (void)
 {
 	const gchar *seat = g_getenv ("XDG_SEAT_PATH");
+	GError *error = NULL;
 	if (seat)  // else, we could possibly get it by: ck -> GetCurrentSession -> session -> GetSeatId
 	{
-		GError *error = NULL;
 		DBusGProxy *pProxy = cairo_dock_create_new_system_proxy (
 			"org.freedesktop.DisplayManager",
 			seat,
@@ -890,6 +883,8 @@ static void cd_logout_switch_to_guest (void)
 		{
 			cd_warning ("DisplayManager error: %s", error->message);
 			g_error_free (error);
+			if (myConfig.cUserActionSwitchUser != NULL)
+				cairo_dock_launch_command (myConfig.cUserActionSwitchUser);
 		}
 		g_object_unref (pProxy);
 	}
@@ -899,9 +894,17 @@ static void cd_logout_switch_to_guest (void)
 			"org.gnome.DisplayManager",
 			"/org/gnome/DisplayManager/LocalDisplayFactory",
 			"org.gnome.DisplayManager.LocalDisplayFactory");
-		dbus_g_proxy_call_no_reply (pProxy, "StartGuestSession",
+		dbus_g_proxy_call (pProxy, "StartGuestSession", &error,
 			G_TYPE_STRING, "",  // current user session, but actually it can be NULL so why bother?
+			G_TYPE_INVALID,
 			G_TYPE_INVALID);  // we don't care the 'id' object path returned
+		if (error)
+		{
+			cd_warning ("DisplayManager (LocalDisplay) error: %s", error->message);
+			g_error_free (error);
+			if (myConfig.cUserActionSwitchUser != NULL)
+				cairo_dock_launch_command (myConfig.cUserActionSwitchUser);
+		}
 		g_object_unref (pProxy);
 	}
 }
